"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SelectionContext = void 0;
const prosemirror_keymap_1 = require("prosemirror-keymap");
const prosemirror_state_1 = require("prosemirror-state");
const nodes_1 = require("../../../utils/nodes");
const tooltip_1 = require("./tooltip");
const SelectionContext = (builder, { config }) => {
    if (Array.isArray(config) && config.length > 0) {
        builder.addPlugin(({ actions }) => new prosemirror_state_1.Plugin(new SelectionTooltip(actions, config)));
    }
};
exports.SelectionContext = SelectionContext;
class SelectionTooltip {
    constructor(actions, menuConfig) {
        this.destroyed = false;
        this.hideTimeoutRef = null;
        this._isMousePressed = false;
        this.tooltip = new tooltip_1.TooltipView(actions, menuConfig);
    }
    get props() {
        return {
            // same as keymap({})
            handleKeyDown: (0, prosemirror_keymap_1.keydownHandler)({
                // hide context menu when Esc was pressed
                Escape: (_state, _dispatch, view) => {
                    if (this.tooltip.isTooltipOpen) {
                        this.tooltip.hide(view);
                        return true;
                    }
                    return false;
                },
            }),
            handleDOMEvents: {
                mousedown: (view) => {
                    this._isMousePressed = true;
                    this.cancelTooltipHiding();
                    this.tooltip.hide(view);
                    const onMouseUp = () => {
                        if (this.destroyed)
                            return;
                        this._isMousePressed = false;
                        this.update(view, this._prevState);
                    };
                    document.addEventListener('mouseup', onMouseUp, { once: true });
                },
            },
        };
    }
    view(view) {
        this.update(view);
        return {
            update: this.update.bind(this),
            destroy: () => {
                this.destroyed = true;
                this.cancelTooltipHiding();
                this.tooltip.destroy();
            },
        };
    }
    update(view, prevState) {
        this._prevState = prevState;
        if (this._isMousePressed)
            return;
        this._prevState = null;
        this.cancelTooltipHiding();
        // Don't show tooltip if editor not mounted to the DOM
        // or when view is out of focus
        if (!view.dom.parentNode || !view.hasFocus()) {
            this.tooltip.hide(view);
            return;
        }
        const { state } = view;
        // Don't do anything if the document/selection didn't change
        if (prevState && prevState.doc.eq(state.doc) && prevState.selection.eq(state.selection))
            return;
        const { selection } = state;
        // Hide the tooltip if the selection is empty
        if (selection.empty || !(selection instanceof prosemirror_state_1.TextSelection)) {
            this.tooltip.hide(view);
            return;
        }
        // Hide the tooltip when one side of selection inside codeblock
        if ((0, nodes_1.isCodeBlock)(selection.$from.parent) || (0, nodes_1.isCodeBlock)(selection.$to.parent)) {
            this.tooltip.hide(view);
            return;
        }
        this.tooltip.show(view, {
            onOutsideClick: () => {
                this.scheduleTooltipHiding(view);
            },
        });
    }
    scheduleTooltipHiding(view) {
        this.hideTimeoutRef = setTimeout(() => {
            // hide tooltip if view is out of focus after 30 ms
            if (!view.hasFocus()) {
                this.tooltip.hide(view);
            }
        }, 30);
    }
    cancelTooltipHiding() {
        if (this.hideTimeoutRef !== null) {
            clearTimeout(this.hideTimeoutRef);
            this.hideTimeoutRef = null;
        }
    }
}
