import { decode as base64ToBuffer } from 'base64-arraybuffer';
import { Fragment, Slice } from 'prosemirror-model';
import { Plugin } from 'prosemirror-state';
import { dropPoint } from 'prosemirror-transform';
import { isFunction } from '../../../../lodash';
import { clipboardUtils } from '../../../behavior/Clipboard';
import { ImageAttr, ImgSizeAttr, imageType } from '../../../specs';
import { isImageNode } from '../utils';
import { ImagesUploadProcess } from './upload';
const { isFilesFromHtml, isFilesOnly, isImageFile } = clipboardUtils;
export const ImagePaste = (builder, opts) => {
    if (!opts || !isFunction(opts.imageUploadHandler))
        throw new Error('ImagePaste extension: imageUploadHandler is not a function');
    builder.addPlugin(() => new Plugin({
        props: {
            handleDOMEvents: {
                paste(view, e) {
                    const files = getPastedImages(e.clipboardData);
                    if (files) {
                        e.preventDefault();
                        new ImagesUploadProcess(view, files, opts.imageUploadHandler, view.state.tr.selection.from, opts).run();
                        return true;
                    }
                    return false;
                },
                drop(view, e) {
                    var _a, _b;
                    // handle drop images from device
                    if (view.dragging)
                        return false;
                    const files = getPastedImages(e.dataTransfer);
                    if (!files)
                        return false;
                    const dropPos = (_b = (_a = view.posAtCoords({ left: e.clientX, top: e.clientY })) === null || _a === void 0 ? void 0 : _a.pos) !== null && _b !== void 0 ? _b : -1;
                    if (dropPos === -1)
                        return false;
                    const posToInsert = dropPoint(view.state.doc, dropPos, createFakeImageSlice(view.state.schema));
                    if (posToInsert !== null) {
                        new ImagesUploadProcess(view, files, opts.imageUploadHandler, posToInsert, opts).run();
                    }
                    e.preventDefault();
                    return true;
                },
            },
            handlePaste(view, _event, slice) {
                const node = sliceSingleNode(slice);
                if (node && isImageNode(node)) {
                    const imgUrl = node.attrs[ImgSizeAttr.Src];
                    const imgFile = dataUrlToFile(imgUrl, 'image');
                    if (imgFile) {
                        new ImagesUploadProcess(view, [imgFile], opts.imageUploadHandler, view.state.tr.selection.from, opts).run();
                        return true;
                    }
                }
                return false;
            },
        },
    }), builder.Priority.High);
};
function getPastedImages(data) {
    if (!data || data.files.length === 0)
        return null;
    if (!isFilesOnly(data) && !isFilesFromHtml(data))
        return null;
    const files = Array.from(data.files);
    return files.every(isImageFile) ? files : null;
}
function createFakeImageSlice(schema) {
    return new Slice(Fragment.from(imageType(schema).create({
        [ImageAttr.Src]: 'fake',
        [ImageAttr.Title]: 'image',
    })), 0, 0);
}
// copied from prosemirror-view input.ts
function sliceSingleNode(slice) {
    return slice.openStart === 0 && slice.openEnd === 0 && slice.content.childCount === 1
        ? slice.content.firstChild
        : null;
}
function dataUrlToFile(dataUrl, filename) {
    var _a;
    const [data, base64String] = dataUrl.split(',');
    if (!data || !base64String)
        return null;
    const mime = (_a = data.match(/^data:(.+);base64/)) === null || _a === void 0 ? void 0 : _a[1];
    if (!mime)
        return null;
    return new File([base64ToBuffer(base64String)], filename, { type: mime });
}
